@@ -1,4 +1,4 @@
-### $Id: zzz.R,v 1.1.1.1 2001/03/23 16:15:26 bates Exp $
+### $Id: zzz.R,v 1.3 2003/12/20 09:05:09 ripley Exp $
 ###
 ###             Read stored Minitab worksheets
 ###
@@ -9,20 +9,19 @@
 ### It is made available under the terms of the GNU General Public
 ### License, version 2, or at your option, any later version,
 ### incorporated herein by reference.
-### 
+###
 ### This program is distributed in the hope that it will be
 ### useful, but WITHOUT ANY WARRANTY; without even the implied
 ### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 ### PURPOSE.  See the GNU General Public License for more
 ### details.
-### 
+###
 ### You should have received a copy of the GNU General Public
 ### License along with this program; if not, write to the Free
 ### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 ### MA 02111-1307, USA
 
-.First.lib <- function(lib, pkg) {
-  library.dynam("foreign", pkg, lib)
-  .C("spss_init", PACKAGE = "foreign");
+.onLoad <- .First.lib <- function(lib, pkg) {
+    library.dynam("foreign", pkg, lib)
+    .C("spss_init", PACKAGE = "foreign")
 }
-
